/* ************************************************************************** */
/*                                       _____           _                    */
/*  libg1m/format/mcs.h                 |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/02 14:15:46                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#ifndef LIBG1M_FORMAT_MCS_H
# define LIBG1M_FORMAT_MCS_H
# include <libg1m.h>
# include <stdint.h>
# pragma pack(1)

/*	MCS is the main filesystem on CASIO calculators. They contain settings,
 *	programs, lists, pictures, captures, matrixes, and other data CASIO
 *	calculators own.
 *	Keep in mind that all numbers in the MCS format are BCD-encoded.
 *	Because of some precision issue (0.3), we'll deliver them to the user as is.
 *
 *	MCS files have a group, a directory, a name and a type. This type depends
 *	on the group it's in. So for example, the 0xFE type in the 'S-SHEET' group
 *	will correspond to a spreadsheet! In an MCS G1M file, files are more or less
 *	regrouped by their group (but sometimes they're not, there are several
 *	'PROGRAM' groups with one element for example).
 *
 *	The `number` field in the Standard Header corresponds to the total number
 *	of MCS files, not group instances!
 *
 *	Each group instance have a header, and here it is: */

struct mcs_subheader {
	/* group name (e.g. "PROGRAM", zero padded) */
	uint8_t intname[16];

	/* subitem count */
	uint32_t subcount;
};

/*	Then follows the files, with their headers and contents.
 *	And the header iiiiiiis: */

struct mcs_fileheader {
	/* the directory name (zero-padded).*/
	uint8_t dirname[8];

	/* the file name (item name) */
	uint8_t filename[8];

	/* the file type. */
	uint8_t filetype;

	/* the size of the part data, without the part header. */
	uint32_t datalength;

	/* some alignment */
	uint8_t align[3];
};

# pragma pack()

/*	Beneath the file header, the file have different structures according to
 *	their group name and MCS type. Pick your poison, once again! */

# include <libg1m/format/mcsfile.h>
#endif /* LIBG1M_FORMAT_MCS_H */
