/* ************************************************************************** */
/*                                       _____           _                    */
/*  libg1m/internals/endian.h           |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/12/20 11:44:12                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#ifndef LIBG1M_INTERNALS_ENDIAN_H
# define LIBG1M_INTERNALS_ENDIAN_H

/* MS-Windows stuff */
# if (defined(_WIN16) || defined(_WIN32) || defined(_WIN64)) \
	&& !defined(__WINDOWS__)
#  define __WINDOWS__
# endif

/*	"endian.h" is not a cross-platform header, so I'll be re-using some work
 *	by Mathias Panzenböck to make this internal header. */

# if defined(__linux__) || defined(__CYGWIN__)
#  include <endian.h>

# elif defined(__APPLE__)
#  include <libkern/OSByteOrder.h>

#  define be16toh(x) OSSwapBigToHostInt16(x)
#  define be32toh(x) OSSwapBigToHostInt32(x)
#  define be64toh(x) OSSwapBigToHostInt64(x)

#  define htobe16(x) OSSwapHostToBigInt16(x)
#  define htobe32(x) OSSwapHostToBigInt32(x)
#  define htobe64(x) OSSwapHostToBigInt64(x)

# elif defined(__OpenBSD__)
#  include <sys/endian.h>

# elif defined(__WINDOWS__)
#  include <winsock2.h>
#  include <sys/param.h>

#  if BYTE_ORDER == LITTLE_ENDIAN
#   define be16toh(x) ntohs(x)
#   define be32toh(x) ntohl(x)
#   define be64toh(x) ntohll(x)

#   define htobe16(x) htons(x)
#   define htobe32(x) htonl(x)
#   define htobe64(x) htonll(x)
#  else
#   define be16toh(x) (x)
#   define be32toh(x) (x)
#   define be64toh(x) (x)

#   define htobe16(x) (x)
#   define htobe32(x) (x)
#   define htobe64(x) (x)
#  endif

# else
#  error Unsupported platform!
# endif

#endif /* LIBG1M_INTERNALS_ENDIAN_H */
