/* ************************************************************************** */
/*                                       _____           _                    */
/*  fontcharacter/character.c           |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/21 13:29:30                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>
#include <string.h>

/**
 *	g1m_mbtofc:
 *	Multi-byte to FONTCHARACTER.
 *
 *	Based on what is said in the fx-9860G SDK.
 *	Let's hope it stays true.
 *
 *	Based on how `mbtowc` works.
 *
 *	@arg	pfc		pointer to the FONTCHARACTER character.
 *	@arg	s		the multi-byte source string.
 *	@arg	n		the size of the destination buffer.
 *	@return			the number of bytes used (-1 if error).
 */

int g1m_mbtofc(FONTCHARACTER *pfc, const char *s, size_t n)
{
	/* null string? */
	if (!s) return (-1);

	/* so that the 'no size = unlimited' (n == 0) case is easy to manage */
	n--;

	/* nul character? */
	if (!*s)
		return (0);

	/* extended char? */
	const unsigned char *us = (const unsigned char*)s;
	if (*us && strchr("\x7F\xF7\xF9\xE5\xE6\xE7", *us)) {
		if (!n) return (-1);
		*pfc = *us;
		*pfc <<= 8;
		*pfc |= *(us + 1);
		return (2);
	}

	/* single-byte character! */
	*pfc = *us;
	return (1);
}

/**
 *	g1m_fctomb:
 *	FONTCHARACTER to multi-byte.
 *
 *	Based on the same document, and on how `wctomb` works.
 *
 *	Do you think a battery has self-consciousness?
 *
 *	@arg	s		pointer to the multi-byte string.
 *	@arg	fc		the FONTCHARACTER character.
 *	@return			the
 */

int g1m_fctomb(char *s, FONTCHARACTER fc)
{
	/* no string? */
	if (!s)
		return (-1);

	/* extended? */
	if (fc > 0xff) {
		*s++ = (fc >> 8) & 0xff;
		*s = fc & 0xff;
		return (2);
	}

	/* normal char. */
	*s = fc;
	return (fc != 0);
}
