/* ************************************************************************** */
/*                                       _____           _                    */
/*  image/monochrome_packed.c           |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/23 07:04:13                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_pixels_from_packed1bit:
 *	Convert packed monochrome formatted pixels to the final 32-bit matrix.
 *
 *	@arg	pixels		the final pixels.
 *	@arg	raw			the raw data.
 *	@arg	width		the width.
 *	@arg	height		the height.
 */

void g1m_pixels_from_packed1bit(uint32_t **pixels, unsigned char *raw,
	int width, int height)
{
	uint32_t *cell = (uint32_t*)&pixels[height];
	int cell_count = width * height;

	/* fill cells */
	int byte = 0x80;
	for (int i = 0; i < cell_count; i++) {
		/* get pixel and iterate */
		int pixel = *raw & byte;
		raw += byte & 1;
		byte = (byte >> 1) | ((byte & 1) << 7);

		/* put pixel */
		*cell++ = (pixel != 0) * 0xffffff;
	}
}

/**
 *	g1m_pixels_to_packed1bit:
 *	Convert 32-bit matrix to packed monochrome pixels.
 *
 *	@arg	dest		the destination buffer.
 *	@arg	pixels		the decoded pixels.
 *	@arg	width		the width.
 *	@arg	height		the height.
 */

void g1m_pixels_to_packed1bit(unsigned char *dest, uint32_t **pixels,
	int width, int height)
{
	uint32_t *cell = (uint32_t*)&pixels[height];
	int cell_count = width * height;

	/* fill */
	int byte = 0x80;
	for (int i = 0; i < cell_count; i++) {
		/* get pixel */
		int pixel = ((*cell++ & 0xffffff) != 0) * byte;
		*dest |= pixel;

		/* iterate */
		dest += byte & 1;
		byte = (byte >> 1) | ((byte & 1) << 7);
	}
}
