/* ************************************************************************** */
/*                                       _____           _                    */
/*  image/packed4bits.c                 |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/12/12 23:59:17                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>

/* colors correspondance */
static const uint32_t colors[16] = {
	[g1m_color_black]   = 0x000000,
	[g1m_color_blue]    = 0x0000ff,
	[g1m_color_green]   = 0x00ff00,
	[g1m_color_cyan]    = 0x00ffff,
	[g1m_color_red]     = 0xff0000,
	[g1m_color_magenta] = 0xff00ff,
	[g1m_color_yellow]  = 0xffff00,
	[g1m_color_white]   = 0xffffff,
	/* RESERVED */
};

/**
 *	g1m_pixels_from_packed4bits:
 *	Convert 4-bits formatted image to the final 32-bit matrix.
 *
 *	@arg	pixels		the final pixels.
 *	@arg	raw			the raw data.
 *	@arg	width		the width.
 *	@arg	height		the height.
 */

void g1m_pixels_from_packed4bits(uint32_t **pixels, unsigned char *raw,
	int width, int height)
{
	uint32_t *cell = (uint32_t*)&pixels[height];
	int cell_count = width * height;

	/* fill cells */
	int mask = 0xf0;
	for (int i = 0; i < cell_count; i++) {
		/* get nibble and iterate */
		int b = (*raw & mask) >> (mask & 4);
		raw += mask & 1;
		mask = ~mask & 0xff;

		/* put pixel */
		*cell++ = colors[b];
	}
}

/**
 *	g1m_pixels_to_packed4bits:
 *	Convert the 32-bit matrix to a 4-bit formatted image.
 *
 *	@arg	dest		the destination buffer.
 *	@arg	pixels		the decoded pixels.
 *	@arg	width		the width.
 *	@arg	height		the height.
 */

void g1m_pixels_to_packed4bits(unsigned char *dest, uint32_t **pixels,
	int width, int height)
{
	(void)width;
	uint32_t *cell = (uint32_t*)&pixels[height];
	int cell_count = width * height;

	/* first of all, clean destination */
	bzero(dest, cell_count / 2 + cell_count % 2);

	/* fill cells */
	int mask = 0xf0;
	for (int i = 0; i < cell_count; i++) {
		/* get color and iterate */
		int color = 0;
		color |= ((0xff0000 & *cell) != 0) << 2;
		color |= ((0x00ff00 & *cell) != 0) << 1;
		color |= ((0x0000ff & *cell++) != 0);

		/* store nibble */
		*dest |= color << mask & 4;
		dest += mask & 1;
		mask = ~mask & 0xff;
	}
}
