/* ************************************************************************** */
/*                                       _____           _                    */
/*  parse/fkey.c                        |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/12/17 00:21:32                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	fkeydup:
 *	"Duplicate" function key.
 *
 *	@arg	fkey		fkey to duplicate.
 *	@return				the "duplicated" function key.
 */

static uint32_t **fkeydup(uint8_t *fkey)
{
	uint32_t **pixels = alloc_pixels(FKEY_WIDTH, FKEY_HEIGHT);
	if (!pixels) return (NULL);
	prepare_pixels(pixels, FKEY_WIDTH, FKEY_HEIGHT)

	g1m_pixels_from_packed1bit(pixels, fkey, FKEY_WIDTH, FKEY_HEIGHT);
	return (pixels);
}

/**
 *	g1m_parse_fkey:
 *	Parse fx function keys files.
 *
 *	@arg	handle		the libg1m handle.
 *	@arg	stream		the stream to parse from.
 *	@return				the error code (0 if ok).
 */

int g1m_parse_fkey(g1m_t *handle, FILE *stream,
	struct standard_header *std)
{
	(void)std;
	/* set handle type */
	handle->type = g1m_type_fkey;

	/* read the subheader */
	DREAD(hd, g1n_subheader)
	uint_fast16_t num = be16toh(hd.fkey_count) + 1;

	/* beat the best! read the rest! */
	size_t data_size = std->filesize - sizeof(struct standard_header)
		- sizeof(struct g1n_subheader);
	uint8_t data[data_size];
	READ(data, data_size)

	/* allocate tab */
	handle->fkeys = malloc(sizeof(uint32_t**) * num);
	handle->count = 0;
	handle->_size = num;
	if (!handle->fkeys) return (g1m_error_alloc);

	/* get the offset table */
	uint16_t *offsets = (uint16_t*)data;
	uint8_t *fkeys = (uint8_t*)(offsets + num);

	/* read the function keys name */
	handle->title[16] = 0;
	strncpy(handle->title, (char*)fkeys, 16);

	/* read all */
	for (uint_fast32_t i = 1; i < num; i++) {
		handle->fkeys[i] = NULL;
		if (offsets[i] == (uint16_t)-1) {
			handle->count++;
			continue ;
		}

		/* correct offset */
		offsets[i] = be16toh(offsets[i]);

		/* store */
		handle->fkeys[i] = fkeydup(&fkeys[i]);
		if (!handle->fkeys[i]) goto fail;
		handle->count++;
	}

	/* no error */
	return (0);

	/* omg fail! */
fail:
	for (int i = 0; i < handle->count; i++)
		free(handle->fkeys[i]);
	free(handle->messages);
	handle->messages = NULL;
	return (g1m_error_alloc);
}
