/* ************************************************************************** */
/*                                       _____           _                    */
/*  user/open.c                         |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/21 09:14:15                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>
#include <libg1m/internals/stdio_ext.h>
#include <stdlib.h>

/**
 *	g1m_open:
 *	Open handle using path.
 *
 *	@arg	handle		the handle to create.
 *	@arg	path		the path of the file to open.
 *	@return				the error code (0 if ok).
 */

int g1m_open(g1m_t **handle, const char *path)
{
	/* open stream */
	FILE *f = fopen(path, "r");

	/* open using `g1m_fopen` */
	int err = g1m_fopen(handle, f);

	/* close opened stream and return error code (or 0 if ok) */
	if (f) fclose(f);
	return (err);
}

/**
 *	g1m_fopen:
 *	Open handle using FILE* pointer.
 *
 *	Will not seek, and will not keep the stream.
 *	Make sure to fclose the stream after use.
 *
 *	@arg	handle		the handle to create
 *	@arg	stream		the stream
 *	@return				the error code (0 if ok)
 */

int g1m_fopen(g1m_t **handle, FILE *stream)
{
	int err;

	/* check stream */
	if (!stream) return (g1m_error_nostream);
	if (!__freadable(stream)) return (g1m_error_noread);

	/* create the handle */
	*handle = malloc(sizeof(g1m_t));
	if (!*handle) return (g1m_error_alloc);

	/* fill it by parsing opened file */
	if ((err = g1m_parse(*handle, stream))) {
		free(*handle);
		*handle = NULL;
		return (err);
	}

	/* everything ok */
	return (0);
}
