/* ************************************************************************** */
/*                                       _____           _                    */
/*  utils/mcstype.c                     |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/12/16 12:26:03                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>

/* ************************************************************************** */
/*  Local types                                                               */
/* ************************************************************************** */
/* Correspondance type */
#define noarg 0
#define arg 1
#define arg_is_num 2
#define arg_is_let 0
#define weight_by_gid 4
struct type_corresp {
	/* identification */
	unsigned int rawtype;
	const char *name; /* NULL: don't check */
	unsigned int flags;

	/* information for logging and re-constituting information */
	char *dirname;
	const char *info;

	/* libg1m type */
	unsigned int type;
};

/* Group correspondance type */
struct group_corresp {
	/* identification */
	const char *name; /* NULL = don't check */
	unsigned int flags;

	/* types */
	struct type_corresp *types;
};

/* ************************************************************************** */
/*  Correspondances                                                           */
/* ************************************************************************** */
/*	All correspondances. Some remarks:
 *	- I think files with "PROGRAM" group are splitted because it was originally
 *	  planned they would be in groups like 'PROGRAM <name>', like for captures
 *	  or lists, but they abandoned that. Why they made such groups, I have no
 *	  idea yet.
 *	- 0xfe is more or less the generic type ID for "data".
 *	- I skipped the 'LISTFILE %d' groups are they are more or less duplicates
 *	  of normal lists.
 */

#define TTERM {0, NULL, 0, NULL, NULL, 0}
static struct group_corresp mcs_groups[] = {
	/* Main Memory */
	{"SETUP", noarg, (struct type_corresp[]){
		{0x14, NULL, 0, "$GLOBAL",
			"setup", g1m_mcstype_setup},
		{0x00, NULL, 0, "$GLOBAL",
			"alpha memory", g1m_mcstype_alphamem},
		TTERM
	}},
	{"ALPHA MEM", noarg, (struct type_corresp[]){
		{0x00, NULL, 0, "$GLOBAL",
			"alpha memory", g1m_mcstype_alphamem},
		TTERM
	}},
	{"CAPT ", arg | arg_is_num, (struct type_corresp[]){
		{0x0A, "CAPT", arg | arg_is_num, "@REV2",
			"capture", g1m_mcstype_capt},
		TTERM
	}},
	{"PICTURE ", arg | arg_is_num, (struct type_corresp[]){
		{0x07, "PICT", arg | arg_is_num, "main",
			"picture", g1m_mcstype_pict},
		TTERM
	}},
	{"STRING ", arg | arg_is_num, (struct type_corresp[]){
		{0x04, "STR", arg | arg_is_num, "main",
			"string", g1m_mcstype_string},
		TTERM
	}},
	{"LIST ", arg | arg_is_num, (struct type_corresp[]){
		{0x05, "1LIST", arg | arg_is_num, "main",
			"list", g1m_mcstype_list},
		TTERM
	}},
	{"LISTFILE ", arg | arg_is_num | weight_by_gid, (struct type_corresp[]){
		{0x05, "1LIST", arg | arg_is_num | weight_by_gid, "main",
			"list", g1m_mcstype_list},
		TTERM
	}},
	{"MAT ", arg | arg_is_let, (struct type_corresp[]){
		{0x06, "MAT_", arg | arg_is_let, "main",
			"matrix", g1m_mcstype_mat},
		TTERM
	}},
	{"VCT ", arg | arg_is_let, (struct type_corresp[]){
		{0x06, "VCT_", arg | arg_is_let, "main",
			"vector", g1m_mcstype_vct},
		TTERM
	}},
	{"PROGRAM", noarg, (struct type_corresp[]){
		{0x01, NULL, 0, "system",
			"program", g1m_mcstype_program},
		TTERM
	}},
	{"Y=DATA", noarg, (struct type_corresp[]){
		{0x02, NULL, 0, "main",
			"unknown", 0x00},
		{0x05, NULL, 0, "main",
			"unknown", 0x00},
		TTERM
	}},
	{"V-WIN ", arg | arg_is_num, (struct type_corresp[]){
		{0x05, "VMEM", arg | arg_is_num, "main",
			"unknown", 0x00},
		TTERM
	}},

	/* application-specific data */
	{"SYSTEM", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@RUNMAT",
			"replay", 0x00},
		{0x0A, NULL, 0, "@REV2",
			"clip", 0x00},
		TTERM
	}},
	{"S-SHEET", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@SSHEET",
			"spreadsheet", g1m_mcstype_spreadsheet},
		TTERM
	}},
	{"CONICS", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@CONICS",
			"conics data", 0x00},
		TTERM
	}},
	{"DYNA MEM", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@DYNA",
			"dyna data", 0x00},
		TTERM
	}},
	{"Econ3Now", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@E-CON2",
			"econ3 data", 0x00},
		TTERM
	}},
	{"FINANCIAL", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@FINANCE",
			"financial data", 0x00},
		TTERM
	}},
	{"RECURSION", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@RECUR",
			"recursion data", 0x00},
		TTERM
	}},
	{"STAT", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@STAT",
			"stat data", 0x00},
		{0x05, "STATV", arg | arg_is_num, "main",
			"stat data 2?", 0x00},
		TTERM
	}},
	{"TABLE", noarg, (struct type_corresp[]){
		{0xFE, NULL, 0, "@TABLE",
			"range data?", 0x00},
		TTERM
	}},

	/* terminating entry */
	{NULL, 0, NULL}
};

/* ************************************************************************** */
/*  Main functions                                                            */
/* ************************************************************************** */
/**
 *	get_number:
 *	Get number from string.
 *
 *	@arg	s			the string.
 *	@arg	num			pointer to the num to fill.
 *	@arg	isnum		is it a number or a letter?
 *	@return				if there was an error (0 if ok).
 */

static int get_number(const char *s, int *num, int isnum)
{
	if (!strcmp(s, "Ans"))
		*num = 27;
	else if (isnum) {
		if (!(*num = atoi(s))) return (1);
	} else {
		if (*s < 'A' || *s > 'Z')
			return (1);
		*num = *s - 'A' + 1;
	}
	return (0);
}


/**
 *	g1m_mcstype_getlib:
 *	Get libg1m type from rawtype.
 *
 *	@arg	groupname	the MCS group name.
 *	@arg	filename	the filename.
 *	@arg	rawtype		the raw number type.
 *	@arg	type		pointer to the real type field to fill.
 *	@arg	id			pointer to the ID to fill.
 *	@return				if the type was not found (0 if yes).
 */

int g1m_mcstype_getlib(const unsigned char *groupname,
	const unsigned char *name, unsigned int rawtype,
	unsigned int *type, int *id)
{
	char gname[17]; gname[16] = 0;
	strncpy(gname, (char*)groupname, 16);
	char fname[9]; fname[8] = 0;
	strncpy(fname, (char*)name, 8);
	*id = 0;

	/* log what we're looking for */
	log_info("Looking for type with '%s' group, '%s' name and 0x%02x raw type",
		gname, fname, rawtype);

	/* look for group correspondance */
	int gid = 0;
	struct group_corresp *g = mcs_groups - 1;
	while ((++g)->types) {
		size_t pl = strlen(g->name);

		/* check if pattern is there */
		if (strncmp(g->name, gname, pl))
			continue;
		if ((g->flags & arg)
		 && get_number(&gname[pl], &gid, g->flags & arg_is_num))
			continue;
		break;
	}
	if (!g->types)
		goto notfound;

	/* look for type correspondance */
	struct type_corresp *t = g->types - 1;
	while ((++t)->dirname) {
		if (t->rawtype != rawtype)
			continue;

		/* check if pattern */
		if (!t->name)
			break;
		size_t fl = strlen(t->name);
		if (strncmp(t->name, fname, fl))
			continue;
		int fid = 0;
		if ((t->flags & arg)
		 && get_number(&fname[fl], &fid, t->flags & arg_is_num))
			continue;
		*id |= fid;
		if (t->flags & weight_by_gid)
			*id |= (gid << 5);
		break;
	}
	if (!t->dirname)
		goto notfound;

	/* fill in info and return */
	if (type) *type = t->type;
	return (0);

notfound:
	log_info("Type with '%s' group, '%s' name and 0x%02x type "
		"wasn't recognized.", gname, fname, rawtype);
	return (1);
}

/**
 *	g1m_mcstype_toraw:
 *	Get raw information for writing.
 *
 *	@arg	type		the libg1m type.
 *	@arg	id			the ID.
 *	@arg	name		the name buffer.
 *	@arg	dirname		the directory name buffer.
 *	@arg	groupname	the groupname buffer.
 *	@return				if an error was encountered.
 */

int g1m_mcstype_toraw(unsigned int type, int id,
	char *name, char *dirname, char *groupname)
{
	struct group_corresp *g;
	struct type_corresp *t;

	for (g = mcs_groups; g->types; g++) {
		for (t = g->types; t->dirname; t++) {
			/* check if the libg1m type corresponds */
			if (t->type != type)
				continue;
			/* check if id is weighted by major */
			if (g1m_get_id_major(id)) {
				if (~t->flags & weight_by_gid)
					continue;
			} else if (t->flags & weight_by_gid)
				continue;
			/* we found the entry! */
			goto found;
		}
	}

	/* not found */
	return (1);

found:
	/* put the group name */
	if (g->flags & arg) {
		int grid = g1m_get_id_major(id);
		if ((t->flags & arg) && ~t->flags & weight_by_gid)
			grid = g1m_get_id_minor(id);

		if (grid == 27)
			sprintf(groupname, "%sAns", g->name);
		else if (g->flags & arg_is_num)
			sprintf(groupname, "%s%d", g->name, grid);
		else
			sprintf(groupname, "%s%c", g->name, grid + 'A' - 1);
	} else
		strcpy(groupname, g->name);

	/* put the directory name */
	strcpy(dirname, t->dirname);

	/* put the filename */
	if (t->flags & arg) {
		int namid = g1m_get_id_minor(id);

		if (namid == 27)
			sprintf(name, "%sAns", t->name);
		else if (t->flags & arg_is_num)
			sprintf(name, "%s%d", t->name, namid);
		else
			sprintf(name, "%s%c", t->name, g1m_get_id_minor(id) + 'A' - 1);
	}

	/* no error */
	return (0);
}
